/*
 * Created on 14 juil. 2006
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package gus05.entity.gus.file.editing.editor2;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Execute;
import gus05.framework.features.Give;
import gus05.framework.features.Graphic;
import gus05.framework.features.Register;
import gus05.framework.resources.ActionBuilder;
import gus05.framework.tools.DefaultSupport;


public class Editor2 extends DefaultSupport implements Entity, Give, Register, Graphic, Execute, ActionListener {

    
    public String getName()				{return "gus.file.editing.editor2";}
    public String getCreationDate()		{return "2006.07.14";}
    
    
    
    private ActionBuilder actionBuilder;
    private Service toolBarHolder;
    private Service fileLabel;
    private Service editorHolder;
    private Service findNextFile;
    
    
    private JPanel pane;
    private JPanel viewPane;
    private JToolBar toolBar;
    
    private File file;
    
    private Action saveAction;
    private Action reloadAction;
    private Action repeatPlayAction;
    private Action playNextAction;
    
    private boolean hasBeenModified = false;
    private boolean playActivated = false;
    private boolean repeating = false;
    private boolean playingNext = false;
    
    
    
    public Editor2() throws Exception
    {
        actionBuilder = (ActionBuilder) Outside.resource(this,"app.actionbuilder");
        findNextFile = Outside.service(this,"gus.file.action.findnextfile");
        toolBarHolder = Outside.service(this,"gus.graphic.toolbar.holder1");
        editorHolder = Outside.service(this,"gus.file.editing.editorholder");
        fileLabel = Outside.service(this,"gus.file.label.filelabel"); 
        
        toolBar = (JToolBar) toolBarHolder.gui();
        
        editorHolder.addActionListener(this);
        
        fileLabel.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e)
            {newFileReceived();}});
        
        
        initActions();
        
        
        
        toolBarHolder.register("filelabel",fileLabel.gui());
        toolBar.addSeparator();
        toolBarHolder.register("save",saveAction);
        toolBarHolder.register("reload",reloadAction);
        toolBarHolder.register("repeat",repeatPlayAction);
        toolBarHolder.register("next",playNextAction);
        hideModifiedState();
        hidePlayingState();
        
        viewPane = (JPanel) editorHolder.gui();
        
        pane = new JPanel(new BorderLayout());
        pane.add(toolBar,BorderLayout.NORTH);
        pane.add(viewPane,BorderLayout.CENTER);
        
        
        Outside.service(this,"app.supportexit").addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {checkModification();}
        });
    }
    
    
    
    
    public void give(Object obj) throws Exception
    {fileLabel.give(obj);}
    
    
    public JComponent gui()
    {return pane;}
    
    
    public void execute() throws Exception
    {editorHolder.execute();}
    
    
    public void register(String key, Object obj) throws Exception
    {editorHolder.register(key,obj);}
    
    
    
    
    
    private void initActions()
    {
        saveAction = actionBuilder.build("FILEACTION_save",new Execute(){
            public void execute() throws Exception {saveFile();}});
        
        reloadAction = actionBuilder.build("FILEACTION_reload",new Execute(){
            public void execute() throws Exception {reloadFile();}});
        
        repeatPlayAction = actionBuilder.build("PLAY_repeat",new Execute(){
            public void execute() throws Exception {repeating = !repeating;}});
        
        playNextAction = actionBuilder.build("PLAY_next",new Execute(){
            public void execute() throws Exception {playingNext = !playingNext;}});
     }
    
    
    
    
    
    
    
    
    
    private void newFileReceived()
    {
        try
        {
            checkModification();
            
            hideModifiedState();
            hasBeenModified = false;
            hidePlayingState();
            playActivated = false;
            
            file = (File) fileLabel.provide();
            editorHolder.give(file);
        }
        catch(Exception e)
        {Outside.err(this,"newFileReceived()",e);}
    }
    
 
    
    
    
    
    
    
    
    
    public void actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if(command.equals("dataModified()"))	dataModified();
        if(command.equals("fileSaved()"))		fileSaved();
        if(command.equals("playStarted()"))		playStarted();
        if(command.equals("playFinished()"))	playFinished();
    }
    
    
    private void dataModified()
    {
        showModifiedState();
        hasBeenModified = true;
    }
    
    
    private void fileSaved()
    {
        hideModifiedState();
        hasBeenModified = false;
    }
    
    
    private void playStarted()
    {
        showPlayingState();
        playActivated = true;
    }
    
    
    private void playFinished()
    {
        if(playingNext) playNext();
        else if(repeating) repeatPlay();
    }
    
    
    
    
    
    
    
    
    
    
    
    private void checkModification()
    {
        if(!hasBeenModified)return;
        
        boolean save = confirmModification("File saving",
                "File "+file.getAbsolutePath()+"\nhas been modified.\nDo you want to save modifications ?");
        if(save) saveFile();
    }
    
    
    
    private boolean confirmModification(String title, String message)
    {
        int val = JOptionPane.showConfirmDialog(null, message,title,
                JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
        return val==JOptionPane.YES_OPTION;
    }
    
    
    
    
    
    private void saveFile()
    {
        try
        {
            editorHolder.register("save",null);
            hideModifiedState();
            hasBeenModified = false;
        }
        catch(Exception e)
        {Outside.err(this,"saveFile()",e);}
    }
    
    
    
    
    
    private void reloadFile()
    {
        try
        {
            hideModifiedState();
            hasBeenModified = false;
            editorHolder.give(file);
        }
        catch(Exception e)
        {Outside.err(this,"reloadFile()",e);}
    }
    
    
    
    private void playNext()
    {
        try
        {
            File nextFile = findNextFile();
            if(nextFile!=null) editorHolder.give(nextFile);
        }
        catch(Exception e)
        {Outside.err(this,"playNext()",e);}
    }
    
    
    private void repeatPlay()
    {
        try{editorHolder.give(file);}
        catch(Exception e)
        {Outside.err(this,"repeatPlay()",e);}
    }
    
    
    
    
    
    private File findNextFile() throws Exception
    {
        if(repeating)findNextFile.give("+*");
        else findNextFile.give("+");
        return (File)findNextFile.transform(file);
    }
    
    
    
    
    
    
    private void showModifiedState()
    {
        try
        {
            toolBarHolder.register("show","save");
            toolBarHolder.register("show","reload");
        }
        catch(Exception e)
        {Outside.err(this,"showModifiedState()",e);}
    }
     
    
    private void hideModifiedState()
    {
        try
        {
            toolBarHolder.register("hide","save");
            toolBarHolder.register("hide","reload");
        }
        catch(Exception e)
        {Outside.err(this,"hideModifiedState()",e);}
    }
    
    
    
    private void showPlayingState()
    {
        try
        {
            toolBarHolder.register("show","repeat");
            toolBarHolder.register("show","next");
        }
        catch(Exception e)
        {Outside.err(this,"showPlayingState()",e);}
    }
     
    
    private void hidePlayingState()
    {
        try
        {
            toolBarHolder.register("hide","repeat");
            toolBarHolder.register("hide","next");
        }
        catch(Exception e)
        {Outside.err(this,"hidePlayingState()",e);}
    }
    
    
    
    
}
