package gus05.entity.gus.security.asymkeys.filetoprivatekey;


import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.DSAPrivateKeySpec;
import java.security.spec.DSAPublicKeySpec;
import java.security.spec.InvalidKeySpecException;
import java.util.Properties;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Transform;



public class FileToPrivateKey implements Entity, Transform {

    
    public String getName() {return "gus.security.asymkeys.filetoprivatekey";}
    public String getCreationDate() {return "2008.06.10";}
    
    
    private Service verifyKeys;
    private Service verifySign;
    
    private KeyFactory kf;
    
    
    public FileToPrivateKey() throws Exception
    {
    	verifyKeys = Outside.service(this,"gus.security.asymkeys.verifykeys");
    	verifySign = Outside.service(this,"gus.security.asymkeys.verifysign");
    	kf = KeyFactory.getInstance(SECURITY.SECURITY_ALGO_DSA);
    }

    
    
    public Object transform(Object obj) throws Exception
	{
    	File file = (File) obj;
    	if(!file.exists()) throw new Exception("private key file not found: "+file.getAbsolutePath());
		
    	file.setReadOnly();
    	String fileName = file.getName();
		Properties prop = prop(file);
		
		if(prop.size()!=SECURITY.PRIVATEKEY_INFONUMBER)
			throw new Exception("wrong properties number: "+prop.size()+" inside file: "+fileName);
		
		String x1 = findValue(prop,SECURITY.PRIVATEKEY_X1);
		String p1 = findValue(prop,SECURITY.PRIVATEKEY_P1);
        String q1 = findValue(prop,SECURITY.PRIVATEKEY_Q1);
        String g1 = findValue(prop,SECURITY.PRIVATEKEY_G1);
        
        String y0 = findValue(prop,SECURITY.PRIVATEKEY_Y0);
        String p0 = findValue(prop,SECURITY.PRIVATEKEY_P0);
        String q0 = findValue(prop,SECURITY.PRIVATEKEY_Q0);
        String g0 = findValue(prop,SECURITY.PRIVATEKEY_G0);
        
        String owner = findValue(prop,SECURITY.PRIVATEKEY_OWNER);
        String sign = findValue(prop,SECURITY.PRIVATEKEY_SIGN);
        
		if(!fileName.equals(owner+"."+SECURITY.PRIVATEKEY_FILEEXTENSION))
        	throw new Exception("wrong owner value: "+owner+" inside file: "+fileName);
		
        
        PrivateKey privateKey = buildPrivateKey(x1,p1,q1,g1);
        PublicKey publicKey = buildPublicKey(y0,p0,q0,g0);
        
        verifyKeys(privateKey,publicKey);
        verifySign(publicKey,owner,sign);
        return privateKey;
	}
    
    
    
    
    
    private PrivateKey buildPrivateKey(String x, String p, String q, String g) throws InvalidKeySpecException
    {
    	DSAPrivateKeySpec keySpec = new DSAPrivateKeySpec(big(x),big(p),big(q),big(g));
        return kf.generatePrivate(keySpec);
    }
    
    private PublicKey buildPublicKey(String y, String p, String q, String g) throws InvalidKeySpecException
    {
    	DSAPublicKeySpec keySpec = new DSAPublicKeySpec(big(y),big(p),big(q),big(g));
        return kf.generatePublic(keySpec);
    }
    
    
    
    
    
    
    private void verifyKeys(PrivateKey privateKey, PublicKey publicKey) throws Exception
    {verifyKeys.give(new Object[]{privateKey,publicKey});}
    
    
    private void verifySign(PublicKey publicKey, String owner, String sign) throws Exception
    {verifySign.give(new Object[]{publicKey,owner,sign});}
    
    
    
    
    
    
    
    private String findValue(Properties prop, String key) throws Exception
    {
    	if(!prop.containsKey(key))
    		throw new Exception("Invalid private key properties: "+key+" is missing");
    	return prop.getProperty(key);
    }
    
    
    
    private Properties prop(File file) throws IOException
    {
        Properties prop = new Properties();
        FileInputStream fis = new FileInputStream(file);
        prop.load(fis);
        fis.close();
        return prop;
    }
    
    private BigInteger big(String value)
    {return new BigInteger(value);}
} 
