package gus05.entity.gus.security.asymkeys.publickey.signverifier;


import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.PublicKey;
import java.security.Signature;

import gus05.framework.core.Entity;
import gus05.framework.core.Outside;
import gus05.framework.core.Service;
import gus05.framework.features.Filter;
import gus05.framework.features.Transform;



public class PublicKeySignVerifier implements Entity, Transform {


    public String getName()				{return "gus.security.asymkeys.publickey.signverifier";}
    public String getCreationDate()		{return "2008.07.01";}

    public static int BUFFER = 2048;
    
    
    private Service base64Decoder;
    private Signature si;
    


    public PublicKeySignVerifier() throws Exception
    {
    	base64Decoder = Outside.service(this,"gus.tobytearray.base64tobyte");
    	si = Signature.getInstance(SECURITY.SECURITY_ALGO_DSA);
    }



    public Object transform(Object obj) throws Exception
    {
    	PublicKey key = (PublicKey) obj;
        return new SignVerifier(key);
    }


    
    
    private class SignVerifier implements Filter
    {
    	private PublicKey key;
    	public SignVerifier(PublicKey key) {this.key = key;}
    	
		public boolean filter(Object obj) throws Exception
		{
			Object[] info = (Object[]) obj;
			if(info.length!=2) throw new Exception("wrong info number: "+info.length);
			
			Object data = info[0];
			String sign = (String) info[1];
			return verifySign(key,data,sign);
		}
    }
    
    
    
    
    
    private boolean verifySign(PublicKey key, Object obj, String sign) throws Exception
    {
    	if(obj instanceof byte[])
    		return verify(key,(byte[])obj,sign);
    	if(obj instanceof String)
    		return verify(key,((String)obj).getBytes(),sign);
    	if(obj instanceof InputStream)
    		return verify(key,(InputStream)obj,sign);
        if(obj instanceof File)
        	return verify(key,new FileInputStream((File)obj),sign);
        
        throw new Exception("Invalid input: "+obj.getClass().getName());
    }
    
    
    
    
    private boolean verify(PublicKey key, byte[] data, String sign) throws Exception
    {
    	si.initVerify(key);
        si.update(data);
        byte[] signData = decode(sign);
        return si.verify(signData);
    }
    
    
    
    
    private boolean verify(PublicKey key, InputStream is, String sign) throws Exception
    {
    	si.initVerify(key);
    	int b;
    	byte[] data = new byte[BUFFER];
        while((b = is.read(data,0,BUFFER))!=-1)
        {si.update(data,0,b);}
        byte[] signData = decode(sign);
        return si.verify(signData);
    }

    
    

    private byte[] decode(String s) throws Exception
    {return (byte[]) base64Decoder.transform(s);}
}

